"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import typing

from .data.version import VersionRange


class Options:
    weighted_versions: bool = False
    version_ranges: typing.Dict[str, VersionRange] = None

    def __init__(self, weighted_versions: bool = False, version_ranges: typing.Dict[str, VersionRange] = None):
        self.weighted_versions = weighted_versions
        if version_ranges is not None:
            self.version_ranges = version_ranges

    def __repr__(self):
        return f"Options(weighted_versions={self.weighted_versions}, version_ranges={self.version_ranges})"



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
from typing import Union

from . import user_agent, options as _options, data as _data


def generate(device: Union[_data.T_DEVICES, tuple, list, None] = None,
             platform: Union[_data.T_PLATFORMS, tuple, list, None] = None,
             browser: Union[_data.T_BROWSERS, tuple, list, None] = None,
             options: Union[_options.Options, None] = None) -> user_agent.UserAgent:
    return user_agent.UserAgent(device, platform, browser, options)



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator
from src.ua_generator import exceptions


class TestExceptions(unittest.TestCase):
    def test_value_error(self):
        def raised_call():
            ua_generator.generate(device='desktop', platform='invalid111')

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_2(self):
        def raised_call():
            ua_generator.generate(browser=('invalid111', 'invalid112'))

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_3(self):
        def raised_call():
            ua_generator.generate(device='invalid111', platform='android', browser='chrome')

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_4(self):
        def raised_call():
            for i in range(0, 100):
                ua_generator.generate(device=('desktop', 'invalid111'))

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_5(self):
        def raised_call():
            for i in range(0, 100):
                ua_generator.generate(platform=('invalid111', 'macos'))

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_6(self):
        def raised_call():
            for i in range(0, 100):
                ua_generator.generate(browser=('invalid111', 'chrome'))

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_7(self):
        def raised_call():
            ua = ua_generator.generate()
            return ua.ch.invalid111

        self.assertRaises(exceptions.InvalidArgumentError, raised_call)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

from src.ua_generator import utils


class TestUtils(unittest.TestCase):
    def test_choice(self):
        self.assertEqual(utils.choice('mobile'), 'mobile')
        self.assertEqual(utils.choice(('mobile', 'mobile')), 'mobile')
        self.assertEqual(utils.choice(['mobile', 'mobile']), 'mobile')
        self.assertEqual(utils.choice(None), None)
        self.assertEqual(utils.choice(111), None)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import random
from typing import Union, List

from .. import utils


class Version:
    major: int = None
    minor: int = None
    build: int = None
    patch: int = None

    def __init__(self,
                 major: Union[int, tuple] = None,
                 minor: Union[int, tuple] = None,
                 build: Union[int, tuple] = None,
                 patch: Union[int, tuple] = None):
        self.major, self.minor, self.build, self.patch = map(
            lambda x:
            # https://docs.python.org/3/tutorial/controlflow.html#tut-unpacking-arguments
            random.randrange(*x) if isinstance(x, tuple) else x,
            (major, minor, build, patch)
        )
        self.__tuple = None

    def format(self, partitions=None, separator='.', trim_zero=False) -> str:
        versions = [self.major, self.minor, self.build, self.patch]

        if partitions is not None:
            versions = versions[:partitions]
        else:
            # Stop at None
            while versions and versions[-1] is None:
                versions.pop()

        # None to zero
        versions = list(part or 0 for part in versions)

        if trim_zero:
            while versions[-1] == 0:
                versions.pop()

        return separator.join(str(part) for part in versions)

    def __str__(self):
        return self.format()

    # We use it for comparison. See: https://docs.python.org/3/reference/expressions.html#comparisons
    def to_tuple(self) -> tuple:
        if self.__tuple is not None:
            return self.__tuple

        self.__tuple = tuple(part or 0 for part in [self.major, self.minor, self.build, self.patch])
        return self.__tuple

    def __eq__(self, other):
        return self.to_tuple() == other.to_tuple()

    def __ne__(self, other):
        return self.to_tuple() != other.to_tuple()

    def __lt__(self, other):
        return self.to_tuple() < other.to_tuple()

    def __gt__(self, other):
        return self.to_tuple() > other.to_tuple()

    def __le__(self, other):
        return self.to_tuple() <= other.to_tuple()

    def __ge__(self, other):
        return self.to_tuple() >= other.to_tuple()


class ChromiumVersion(Version):
    webkit: Version = None

    def __init__(self, version: Version, webkit: Version = Version(major=537, minor=36)):
        super().__init__(version.major, version.minor, version.build, version.patch)
        self.webkit = webkit


class AndroidVersion(Version):
    build_number: str = None
    platform_model: str = None

    def __init__(self, version: Version, build_numbers: Union[str, tuple, list, None] = None):
        super().__init__(version.major, version.minor, version.build, version.patch)
        self.build_number = utils.choice(build_numbers)


class WindowsVersion(Version):
    ch_platform: Version = None

    def __init__(self, version: Version, ch_platform: Version):
        super().__init__(version.major, version.minor, version.build, version.patch)
        self.ch_platform = ch_platform


VERSION_TYPES = (
    Version,
    ChromiumVersion,
    AndroidVersion,
    WindowsVersion,
)


class VersionRange:
    min_version: Version = None
    max_version: Version = None

    def __init__(self, min_version: Union[Version, int] = None, max_version: Union[Version, int] = None):
        self.min_version = Version(major=min_version) if type(min_version) is int else min_version
        self.max_version = Version(major=max_version) if type(max_version) is int else max_version

    def filter(self, versions: List[Version]) -> List[Version]:
        tmp_versions: List[Version] = []

        # TODO: Perhaps support for full version comparison, instead of just major versions
        for version in versions:
            if self.min_version is not None and self.max_version is not None and self.min_version.major <= version.major <= self.max_version.major:
                tmp_versions.append(version)
            elif self.min_version is not None and self.max_version is None and self.min_version.major <= version.major:
                tmp_versions.append(version)
            elif self.min_version is None and self.max_version is not None and version.major <= self.max_version.major:
                tmp_versions.append(version)

        return tmp_versions

    def __repr__(self):
        return f"VersionRange(min_version={self.min_version}, max_version={self.max_version})"



"""
Random User-Agent
Copyright: 2022-2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import random

from . import android_samsung, android_pixel, android_xiaomi, android_oppo
from ...version import AndroidVersion
from ....options import Options


def get_version(options: Options) -> AndroidVersion:
    choice = random.randint(0, 20)

    if choice < 2:
        return android_pixel.get_version(options=options)
    if choice < 4:
        return android_oppo.get_version(options=options)
    if choice < 10:
        return android_xiaomi.get_version(options=options)

    return android_samsung.get_version(options=options)



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator


class TestPlatform(unittest.TestCase):
    def test_platform(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('linux',))
            self.assertTrue(ua.platform == 'linux')

    def test_platform_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('windows', 'macos'))
            self.assertTrue(ua.platform == 'windows' or ua.platform == 'macos')

    def test_platform_3(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='macos')
            self.assertTrue(ua.platform == 'macos')

    def test_platform_4(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='mobile')
            self.assertTrue(ua.platform == 'ios' or ua.platform == 'android')

    def test_platform_5(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='desktop')
            self.assertTrue(ua.platform == 'windows' or ua.platform == 'macos' or ua.platform == 'linux')

    def test_platform_6(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='desktop', platform='ios')
            self.assertNotEqual(ua.device, 'desktop')
            self.assertEqual(ua.platform, 'ios')

    def test_platform_7(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='desktop', platform='android')
            self.assertNotEqual(ua.device, 'desktop')
            self.assertEqual(ua.platform, 'android')

    def test_platform_8(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='mobile', platform='windows')
            self.assertNotEqual(ua.device, 'mobile')
            self.assertEqual(ua.platform, 'windows')

    def test_platform_9(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='mobile', platform='macos')
            self.assertNotEqual(ua.device, 'mobile')
            self.assertEqual(ua.platform, 'macos')


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..version import Version, ChromiumVersion, VersionRange
from ...options import Options

# https://chromereleases.googleblog.com/search/label/Stable%20updates
versions: List[ChromiumVersion] = [
    ChromiumVersion(Version(major=100, minor=0, build=4896, patch=(0, 255))),
    ChromiumVersion(Version(major=101, minor=0, build=4951, patch=(0, 255))),
    ChromiumVersion(Version(major=102, minor=0, build=5005, patch=(0, 255))),
    ChromiumVersion(Version(major=103, minor=0, build=5060, patch=(0, 255))),
    ChromiumVersion(Version(major=104, minor=0, build=5112, patch=(0, 255))),
    ChromiumVersion(Version(major=105, minor=0, build=5195, patch=(0, 255))),
    ChromiumVersion(Version(major=106, minor=0, build=5249, patch=(0, 255))),
    ChromiumVersion(Version(major=107, minor=0, build=5304, patch=(0, 255))),
    ChromiumVersion(Version(major=108, minor=0, build=5359, patch=(0, 255))),
    ChromiumVersion(Version(major=109, minor=0, build=5414, patch=(0, 255))),
    ChromiumVersion(Version(major=110, minor=0, build=5481, patch=(0, 255))),
    ChromiumVersion(Version(major=111, minor=0, build=5563, patch=(0, 255))),
    ChromiumVersion(Version(major=112, minor=0, build=5615, patch=(0, 255))),
    ChromiumVersion(Version(major=114, minor=0, build=5735, patch=(0, 255))),
    ChromiumVersion(Version(major=115, minor=0, build=5790, patch=(0, 255))),
    ChromiumVersion(Version(major=116, minor=0, build=5845, patch=(0, 255))),
    ChromiumVersion(Version(major=117, minor=0, build=5938, patch=(0, 255))),
    ChromiumVersion(Version(major=118, minor=0, build=5993, patch=(0, 255))),
    ChromiumVersion(Version(major=119, minor=0, build=6045, patch=(0, 255))),
    ChromiumVersion(Version(major=120, minor=0, build=6099, patch=(0, 255))),
    ChromiumVersion(Version(major=121, minor=0, build=6167, patch=(0, 255))),
    ChromiumVersion(Version(major=122, minor=0, build=6261, patch=(0, 255))),
    ChromiumVersion(Version(major=123, minor=0, build=6312, patch=(0, 255))),
    ChromiumVersion(Version(major=124, minor=0, build=6367, patch=(0, 255))),
    ChromiumVersion(Version(major=125, minor=0, build=6422, patch=(0, 255))),
    ChromiumVersion(Version(major=126, minor=0, build=6478, patch=(0, 255))),
    ChromiumVersion(Version(major=127, minor=0, build=6533, patch=(0, 255))),
    ChromiumVersion(Version(major=128, minor=0, build=6613, patch=(0, 255))),
    ChromiumVersion(Version(major=129, minor=0, build=6668, patch=(0, 255))),
    ChromiumVersion(Version(major=130, minor=0, build=6723, patch=(0, 255))),
    ChromiumVersion(Version(major=131, minor=0, build=6778, patch=(0, 255))),
    ChromiumVersion(Version(major=132, minor=0, build=6834, patch=(0, 255))),
    ChromiumVersion(Version(major=133, minor=0, build=6943, patch=(0, 255))),
    ChromiumVersion(Version(major=134, minor=0, build=6998, patch=(0, 255))),
]


def get_version(options: Options) -> ChromiumVersion:
    if options.version_ranges is not None and 'chrome' in options.version_ranges:
        if type(options.version_ranges['chrome']) == VersionRange:
            filtered = options.version_ranges['chrome'].filter(versions)
            if type(filtered) == list and len(filtered) > 0:
                return random.choice(filtered)

    weights = None
    if options.weighted_versions:
        weights = [1.0] * len(versions)
        weights[-1] = 10.0
        weights[-2] = 9.0
        weights[-3] = 8.0

    choice: List[ChromiumVersion] = random.choices(versions, weights=weights, k=1)
    return choice[0]



"""
Random User-Agent
Copyright: 2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

from src.ua_generator import serialization


class TestSerialization(unittest.TestCase):
    def test_ch_brand_list(self):
        brand_list = [
            {'brand': 'Not A(Brand', 'version': '99'},
            {'brand': 'UA-Generator', 'version': '0'},
        ]
        self.assertEqual(serialization.ch_brand_list(brand_list), '"Not A(Brand";v="99", "UA-Generator";v="0"')

    def test_ch_bool(self):
        self.assertEqual(serialization.ch_bool(True), '?1')
        self.assertEqual(serialization.ch_bool(False), '?0')

    def test_ch_string(self):
        self.assertEqual(serialization.ch_string('foo'), '"foo"')


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..version import Version, VersionRange
from ...options import Options

# https://www.mozilla.org/en-US/firefox/releases/
versions: List[Version] = [
    Version(major=108, minor=0, build=(0, 2)),
    Version(major=109, minor=0, build=(0, 1)),
    Version(major=110, minor=0, build=(0, 1)),
    Version(major=111, minor=0, build=(0, 1)),
    Version(major=112, minor=0, build=(0, 2)),
    Version(major=113, minor=0, build=(0, 2)),
    Version(major=114, minor=0, build=(0, 2)),
    Version(major=115, minor=0, build=(0, 3)),
    Version(major=115, minor=1, build=0),
    Version(major=115, minor=2, build=(0, 1)),
    Version(major=115, minor=3, build=(0, 1)),
    Version(major=115, minor=4, build=0),
    Version(major=115, minor=5, build=0),
    Version(major=115, minor=6, build=0),
    Version(major=115, minor=7, build=0),
    Version(major=115, minor=8, build=0),
    Version(major=116, minor=0, build=(0, 3)),
    Version(major=117, minor=0, build=(0, 1)),
    Version(major=118, minor=0, build=(0, 2)),
    Version(major=119, minor=0, build=(0, 1)),
    Version(major=120, minor=0, build=(0, 1)),
    Version(major=121, minor=0, build=(0, 1)),
    Version(major=122, minor=0, build=(0, 1)),
    Version(major=123, minor=0, build=(0, 1)),
    Version(major=124, minor=0, build=(0, 2)),
    Version(major=125, minor=0, build=(1, 3)),
    Version(major=126, minor=0, build=0),
    Version(major=127, minor=0, build=(0, 2)),
    Version(major=128, minor=0, build=(0, 3)),
    Version(major=128, minor=1, build=0),
    Version(major=128, minor=2, build=0),
    Version(major=128, minor=3, build=0),
    Version(major=128, minor=4, build=0),
    Version(major=128, minor=5, build=(0, 2)),
    Version(major=128, minor=6, build=0),
    Version(major=129, minor=0, build=(0, 2)),
    Version(major=130, minor=0, build=(0, 1)),
    Version(major=131, minor=0, build=(0, 3)),
    Version(major=132, minor=0, build=(0, 2)),
    Version(major=133, minor=0, build=(0, 3)),
    Version(major=134, minor=0, build=(0, 2)),
    Version(major=135, minor=0, build=(0, 1)),
    Version(major=136, minor=0, build=0),
]


def get_version(options: Options) -> Version:
    if options.version_ranges is not None and 'firefox' in options.version_ranges:
        if type(options.version_ranges['firefox']) == VersionRange:
            filtered = options.version_ranges['firefox'].filter(versions)
            if type(filtered) == list and len(filtered) > 0:
                return random.choice(filtered)

    weights = None
    if options.weighted_versions:
        weights = [1.0] * len(versions)
        weights[-1] = 10.0
        weights[-2] = 9.0
        weights[-3] = 8.0

    choice: List[Version] = random.choices(versions, weights=weights, k=1)
    return choice[0]



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator
from src.ua_generator import serialization
from src.ua_generator.data import BROWSERS_SUPPORT_CH


class TestClientHints(unittest.TestCase):
    def test_ch_platform(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH, platform='macos')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.platform) is str)
            self.assertEqual(ua.ch.platform, '"macOS"')
            self.assertEqual(ua.ch.get_platform(), 'macOS')

    def test_ch_platform_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH, platform='linux')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.platform) is str)
            self.assertEqual(ua.ch.platform, '"Linux"')
            self.assertEqual(ua.ch.get_platform(), 'Linux')

    def test_ch_platform_version(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.platform_version) is str)
            self.assertEqual(ua.ch.platform_version, serialization.ch_string(ua.ch.get_platform_version()))

    def test_ch_platform_version_windows(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.platform_version) is str)
            self.assertEqual(ua.ch.platform_version, serialization.ch_string(ua.ch.get_platform_version()))

    def test_ch_mobile(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH, platform='android')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.mobile) is str)
            self.assertEqual(ua.ch.mobile, '?1')
            self.assertTrue(type(ua.ch.get_mobile()) is bool)
            self.assertEqual(ua.ch.get_mobile(), True)
            self.assertTrue(ua.ch.model is not None and ua.ch.model != "")
            self.assertTrue(ua.ch.get_model() is not None and ua.ch.get_model() != "")

    def test_ch_non_mobile(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH, platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.mobile) is str)
            self.assertEqual(ua.ch.mobile, '?0')
            self.assertTrue(type(ua.ch.get_mobile()) is bool)
            self.assertEqual(ua.ch.get_mobile(), False)

    def test_ch_brands(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser='chrome', platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.brands) is str)
            self.assertTrue(ua.ch.brands.startswith('"Not A(Brand";v="99"'))
            self.assertTrue('Chromium' in ua.ch.brands)
            self.assertTrue('Google Chrome' in ua.ch.brands)
            self.assertTrue(type(ua.ch.get_brands()) is list)
            self.assertEqual(ua.ch.brands, serialization.ch_brand_list(ua.ch.get_brands()))

    def test_ch_brands_full_version_list(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser='edge', platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.brands_full_version_list) is str)
            self.assertTrue(ua.ch.brands_full_version_list.startswith('"Not A(Brand";v="99.0.0.0"'))
            self.assertTrue('Chromium' in ua.ch.brands_full_version_list)
            self.assertTrue('Microsoft Edge' in ua.ch.brands_full_version_list)
            self.assertTrue(type(ua.ch.get_brands(full_version_list=True)) is list)
            self.assertEqual(ua.ch.brands_full_version_list, serialization.ch_brand_list(ua.ch.get_brands(full_version_list=True)))

    def test_ch_bitness(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.bitness) is str)
            self.assertIn(ua.ch.bitness, ('"32"', '"64"'))
            self.assertIn(ua.ch.get_bitness(), ('32', '64'))

    def test_ch_architecture(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.architecture) is str)
            self.assertIn(ua.ch.architecture, ('"arm"', '"x86"'))
            self.assertIn(ua.ch.get_architecture(), ('arm', 'x86'))

    def test_ch_model(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='android', browser='chrome')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.model) is str)
            self.assertTrue(ua.ch.model != '""')
            self.assertTrue(len(ua.ch.model) > 2)
            self.assertEqual(ua.ch.model, serialization.ch_string(ua.ch.get_model()))

    def test_ch_model_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='linux', browser='firefox')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.model) is str)
            self.assertTrue(ua.ch.model == '""')
            self.assertEqual(ua.ch.model, serialization.ch_string(ua.ch.get_model()))

    def test_ch_wow64(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.wow64) is str)
            self.assertEqual(ua.ch.wow64, '?1')
            self.assertTrue(type(ua.ch.get_wow64()) is bool)
            self.assertEqual(ua.ch.get_wow64(), True)

    def test_ch_wow64_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='linux')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.wow64) is str)
            self.assertEqual(ua.ch.wow64, '?0')
            self.assertTrue(type(ua.ch.get_wow64()) is bool)
            self.assertEqual(ua.ch.get_wow64(), False)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""


class CannotGenerateError(Exception):
    pass


class InvalidArgumentError(Exception):
    pass



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import re
import unittest

import src.ua_generator as ua_generator


class TestUserAgent(unittest.TestCase):
    def test_user_agent(self):
        for i in range(0, 100):
            ua = ua_generator.generate()
            self.assertIsNotNone(ua.device)
            self.assertIsNotNone(ua.platform)
            self.assertIsNotNone(ua.browser)
            self.assertIsNotNone(ua.text)

    def test_user_agent_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('ios', 'macos'), browser='safari')
            self.assertIsNotNone(ua.device)
            self.assertIsNotNone(ua.platform)
            self.assertIsNotNone(ua.browser)
            self.assertIsNotNone(ua.text)

    def test_user_agent_3(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='mobile', browser=('safari', 'chrome'))
            self.assertIsNotNone(ua.device)
            self.assertIsNotNone(ua.platform)
            self.assertIsNotNone(ua.browser)
            self.assertIsNotNone(ua.text)

    def test_user_agent_4(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='desktop', browser='firefox')
            self.assertIsNotNone(ua.device)
            self.assertIsNotNone(ua.platform)
            self.assertIsNotNone(ua.browser)
            self.assertIsNotNone(ua.text)

    def test_user_agent_5(self):
        for i in range(0, 1000):
            device = ('desktop', 'mobile')
            platform = ('windows', 'macos', 'ios', 'linux', 'android')
            browser = ('chrome', 'edge', 'firefox', 'safari')
            ua = ua_generator.generate(device=device, browser=browser, platform=platform)
            self.assertIsNotNone(ua.device)
            self.assertIsNotNone(ua.platform)
            self.assertIsNotNone(ua.browser)
            self.assertIsNotNone(ua.text)

    def test_user_agent_not_contains_brackets(self):
        brackets = re.compile('{(d|s|v|build|chrome|firefox|safari|webkit|windows|android|ios|macos|linux)}')
        for i in range(0, 200):
            ua = ua_generator.generate()
            self.assertNotRegex(ua.text, brackets)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..version import Version, VersionRange
from ...options import Options

# User agent cap on macOS
# https://groups.google.com/a/chromium.org/g/blink-dev/c/hAI4QoX6rEo/m/qQNPThr0AAAJ

# https://developer.apple.com/news/releases/
# https://support.apple.com/en-us/HT201222
versions: List[Version] = [
    Version(major=10, minor=11, build=(0, 6)),
    Version(major=10, minor=12, build=(0, 6)),
    Version(major=10, minor=13, build=(0, 6)),
    Version(major=10, minor=14, build=(0, 6)),
    Version(major=10, minor=15, build=(0, 7)),
    Version(major=11, minor=0, build=0),
    Version(major=11, minor=2, build=(0, 3)),
    Version(major=11, minor=3, build=(0, 1)),
    Version(major=11, minor=5, build=(0, 2)),
    Version(major=11, minor=6, build=(0, 6)),
    Version(major=12, minor=0, build=(0, 1)),
    Version(major=12, minor=2, build=(0, 1)),
    Version(major=12, minor=3, build=(0, 1)),
    Version(major=12, minor=4, build=0),
    Version(major=12, minor=5, build=(0, 1)),
    Version(major=12, minor=6, build=(0, 4)),
    Version(major=12, minor=7, build=(0, 5)),
    Version(major=13, minor=0, build=(0, 1)),
    Version(major=13, minor=1, build=0),
    Version(major=13, minor=2, build=(0, 1)),
    Version(major=13, minor=3, build=(0, 1)),
    Version(major=13, minor=4, build=(0, 1)),
    Version(major=13, minor=5, build=(0, 2)),
    Version(major=14, minor=0, build=(0, 1)),
    Version(major=14, minor=1, build=(0, 2)),
    Version(major=14, minor=2, build=(0, 1)),
    Version(major=14, minor=3, build=(0, 1)),
    Version(major=14, minor=4, build=(0, 1)),
    Version(major=14, minor=5, build=0),
    Version(major=14, minor=6, build=(0, 1)),
    Version(major=15, minor=0, build=(0, 1)),
    Version(major=15, minor=1, build=(0, 1)),
    Version(major=15, minor=2, build=(0, 1)),
    Version(major=15, minor=3, build=(0, 1)),
]


def get_version(options: Options) -> Version:
    if options.version_ranges is not None and 'macos' in options.version_ranges:
        if type(options.version_ranges['macos']) == VersionRange:
            filtered = options.version_ranges['macos'].filter(versions)
            if type(filtered) == list and len(filtered) > 0:
                return random.choice(filtered)

    weights = None
    if options.weighted_versions:
        weights = [1.0] * len(versions)
        weights[-1] = 10.0
        weights[-2] = 9.0
        weights[-3] = 8.0

    choice: List[Version] = random.choices(versions, weights=weights, k=1)
    return choice[0]



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator


class TestBrowser(unittest.TestCase):
    def test_browser(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome',))
            self.assertTrue(ua.browser == 'chrome')

    def test_browser_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'firefox'))
            self.assertTrue(ua.browser == 'chrome' or ua.browser == 'firefox')

    def test_browser_3(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('macos', 'ios'), browser='safari')
            self.assertTrue(ua.browser == 'safari')

    def test_browser_4(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('windows', 'linux'), browser='safari')
            self.assertTrue(ua.browser == 'chrome')

    def test_browser_5(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('macos', 'linux'), browser='safari')
            self.assertTrue((ua.platform == 'macos' and ua.browser == 'safari') or
                            (ua.platform == 'linux' and ua.browser == 'chrome'))


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import random
from typing import List

from ...version import Version, AndroidVersion
from ....options import Options

# https://en.wikipedia.org/wiki/Android_version_history
# https://source.android.com/setup/start/build-numbers
versions: List[AndroidVersion] = [
    AndroidVersion(Version(major=11, minor=0, build=0), build_numbers=('RD2A.{d}.{v}', 'RQ3A.{d}.{v}', 'RQ2A.{d}.{v}', 'RQ1D.{d}.{v}', 'RQ1C.{d}.{v}', 'RQ1A.{d}.{v}', 'RD1B.{d}.{v}', 'RD1A.{d}.{v}', 'RP1A.{d}.{v}')),
    AndroidVersion(Version(major=12, minor=0, build=0), build_numbers=('SP1A.{d}.{v}', 'SQ1D.{d}.{v}', 'SD1A.{d}.{v}')),
    AndroidVersion(Version(major=12, minor=1, build=0), build_numbers=('SP2A.{d}.{v}', 'SD2A.{d}.{v}', 'SQ3A.{d}.{v}')),
    AndroidVersion(Version(major=13, minor=0, build=0), build_numbers=('TQ3A.{d}.{v}', 'TQ3C.{d}.{v}', 'TQ2B.{d}.{v}', 'TD4A.{d}.{v}', 'TQ2A.{d}.{v}', 'TQ1A.{d}.{v}', 'TD1A.{d}.{v}', 'TP1A.{d}.{v}')),
    AndroidVersion(Version(major=14, minor=0, build=0), build_numbers=('AP2A.{d}.{v}', 'AD1A.{d}.{v}', 'UD2A.{d}.{v}', 'UQ1A.{d}.{v}', 'UP1A.{d}.{v}', 'UD1A.{d}.{v}')),
    AndroidVersion(Version(major=15, minor=0, build=0), build_numbers=('AP4A.{d}.{v}', 'AP3A.{d}.{v}')),
]

platform_models = ('CH1933', 'CPH2195', 'CPH2263', 'CPH1941', 'CPH2021', 'CPH2211GDPR',
    'CPH2023', 'CPH2009', 'CPH2025', 'CPH2207', 'CPH2173', 'PEEM00',
    'CPH2307', 'CPH2305', 'CPH1917', 'Global', 'Global', 'CPH2125GDPR',
    'CPH2145', 'Global', 'PEGM10', 'CPH1951', 'CPH2089', 'CPH2065',
    'CPH2251', 'CPH2371', )


def get_version(options: Options) -> AndroidVersion:
    weights = None
    if options.weighted_versions:
        weights = [1.0] * len(versions)
        weights[-1] = 10.0
        weights[-2] = 9.0

    choice: List[AndroidVersion] = random.choices(versions, weights=weights, k=1)

    build_number = choice[0].build_number
    build_number = build_number.replace('{d}', '{:02d}{:02d}{:02d}'.format(random.randint(22, 25), random.randint(0, 12), random.randint(0, 29)))
    build_number = build_number.replace('{v}', '{}'.format(random.randint(1, 255)))

    choice[0].build_number = build_number
    choice[0].platform_model = random.choice(platform_models)
    return choice[0]



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..version import Version, WindowsVersion, VersionRange
from ...options import Options

# https://learn.microsoft.com/en-us/windows/win32/sysinfo/operating-system-version
# https://learn.microsoft.com/en-us/microsoft-edge/web-platform/how-to-detect-win11
versions: List[WindowsVersion] = [
    WindowsVersion(Version(major=6, minor=1), ch_platform=Version(major=0)),
    WindowsVersion(Version(major=6, minor=2), ch_platform=Version(major=0)),
    WindowsVersion(Version(major=6, minor=3), ch_platform=Version(major=0)),
    WindowsVersion(Version(major=10, minor=0), ch_platform=Version(major=10)),
    WindowsVersion(Version(major=10, minor=0), ch_platform=Version(major=(13, 15))),
]


def get_version(options: Options) -> WindowsVersion:
    if options.version_ranges is not None and 'windows' in options.version_ranges:
        if type(options.version_ranges['windows']) == VersionRange:
            filtered = options.version_ranges['windows'].filter(versions)
            if type(filtered) == list and len(filtered) > 0:
                return random.choice(filtered)

    weights = None
    if options.weighted_versions:
        weights = [1.0] * len(versions)
        # https://gs.statcounter.com/os-version-market-share/windows/desktop/worldwide
        weights[-1] = 7.0
        weights[-2] = 10.0

    choice: List[WindowsVersion] = random.choices(versions, weights=weights, k=1)
    return choice[0]



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
from .browsers import chrome, safari, firefox, edge
from .platforms import ios, android, linux, windows, macos
from .. import utils, exceptions
from ..options import Options


class Generator:
    def __init__(self, device, platform, browser, options: Options):
        self.device = device
        self.platform = platform
        self.browser = browser
        self.options = options

        self.platform_version = self.__platform_version()
        self.browser_version = self.__browser_version()
        self.user_agent = self.__user_agent()

    def __platform_version(self):
        if self.platform == 'windows':
            return windows.get_version(options=self.options)
        elif self.platform == 'macos':
            return macos.get_version(options=self.options)
        elif self.platform == 'ios':
            return ios.get_version(options=self.options)
        elif self.platform == 'linux':
            return linux.get_version(options=self.options)
        elif self.platform == 'android':
            return android.get_version(options=self.options)

    def __browser_version(self):
        if self.browser == 'chrome':
            return chrome.get_version(options=self.options)
        elif self.browser == 'safari':
            return safari.get_version(options=self.options)
        elif self.browser == 'firefox':
            return firefox.get_version(options=self.options)
        elif self.browser == 'edge':
            return edge.get_version(options=self.options)

    def __user_agent(self):
        if self.platform == 'windows':
            if self.browser == 'chrome':
                template = utils.choice((
                    'Mozilla/5.0 (Windows NT {windows}; Win64; x64) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit}',
                    'Mozilla/5.0 (Windows NT {windows}; WOW64) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit}'
                ))
                template = template.replace('{windows}', str(self.platform_version))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'edge':
                template = utils.choice((
                    'Mozilla/5.0 (Windows NT {windows}; Win64; x64) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit} Edg/{chrome}',
                ))
                template = template.replace('{windows}', str(self.platform_version))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'firefox':
                template = utils.choice((
                    'Mozilla/5.0 (Windows NT {windows}; Win64; x64; rv:{firefox}) Gecko/20100101 Firefox/{firefox}',
                    'Mozilla/5.0 (Windows NT {windows}; WOW64; rv:{firefox}) Gecko/20100101 Firefox/{firefox}',
                ))
                template = template.replace('{windows}', str(self.platform_version))
                template = template.replace('{firefox}', str(self.browser_version))
                return template

        elif self.platform == 'linux':
            if self.browser == 'chrome':
                template = utils.choice((
                    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit}',
                    'Mozilla/5.0 (X11; Ubuntu; Linux x86_64) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit}',
                ))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'edge':
                template = utils.choice((
                    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit} Edg/{chrome}',
                ))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'firefox':
                template = utils.choice((
                    'Mozilla/5.0 (X11; Linux x86_64; rv:{firefox}) Gecko/20100101 Firefox/{firefox}',
                    'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:{firefox}) Gecko/20100101 Firefox/{firefox}',
                ))
                template = template.replace('{firefox}', str(self.browser_version))
                return template

        elif self.platform == 'android':
            if self.browser == 'chrome':
                template = 'Mozilla/5.0 (Linux; Android {android}{model}{build}) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{chrome} Mobile Safari/{webkit}'
                template = template.replace('{android}', str(self.platform_version.major))
                if self.platform_version.platform_model is not None:
                    template = template.replace('{model}', '; ' + self.platform_version.platform_model)
                if self.platform_version.build_number is not None:
                    template = template.replace('{build}', '; Build/' + self.platform_version.build_number)
                template = template.replace('{build}', '').replace('{model}', '')
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'edge':
                template = 'Mozilla/5.0 (Linux; Android {android}{model}{build}) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{chrome} Mobile Safari/{webkit} EdgA/{chrome}'
                template = template.replace('{android}', str(self.platform_version.major))
                if self.platform_version.platform_model is not None:
                    template = template.replace('{model}', '; ' + self.platform_version.platform_model)
                if self.platform_version.build_number is not None:
                    template = template.replace('{build}', '; Build/' + self.platform_version.build_number)
                template = template.replace('{build}', '').replace('{model}', '')
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'firefox':
                template = 'Mozilla/5.0 (Android {android}; Mobile; rv:{firefox}) Gecko/{firefox} Firefox/{firefox}'
                template = template.replace('{android}', str(self.platform_version.major))
                template = template.replace('{firefox}', str(self.browser_version))
                return template

        elif self.platform == 'macos':
            if self.browser == 'chrome':
                template = 'Mozilla/5.0 (Macintosh; Intel Mac OS X {macos}) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit}'
                template = template.replace('{macos}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'edge':
                template = 'Mozilla/5.0 (Macintosh; Intel Mac OS X {macos}) AppleWebKit/{webkit} (KHTML, like Gecko) Chrome/{chrome} Safari/{webkit} Edg/{chrome}'
                template = template.replace('{macos}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'safari':
                template = 'Mozilla/5.0 (Macintosh; Intel Mac OS X {macos}) AppleWebKit/{webkit} (KHTML, like Gecko) Version/{safari} Safari/{webkit}'
                template = template.replace('{macos}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{safari}', str(self.browser_version))
                return template
            if self.browser == 'firefox':
                template = 'Mozilla/5.0 (Macintosh; Intel Mac OS X {macos}; rv:{firefox}) Gecko/20100101 Firefox/{firefox}'
                template = template.replace('{macos}', self.platform_version.format(partitions=2, trim_zero=True))
                template = template.replace('{firefox}', self.browser_version.format(partitions=2))
                return template

        elif self.platform == 'ios':
            if self.browser == 'chrome':
                template = 'Mozilla/5.0 (iPhone; CPU iPhone OS {ios} like Mac OS X) AppleWebKit/{webkit} (KHTML, like Gecko) CriOS/{chrome} Mobile/15E148 Safari/{webkit}'
                template = template.replace('{ios}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'edge':
                template = 'Mozilla/5.0 (iPhone; CPU iPhone OS {ios} like Mac OS X) AppleWebKit/{webkit} (KHTML, like Gecko) Version/15.0 EdgiOS/{chrome} Mobile/15E148 Safari/{webkit}'
                template = template.replace('{ios}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{chrome}', str(self.browser_version))
                return template
            if self.browser == 'safari':
                template = 'Mozilla/5.0 (iPhone; CPU iPhone OS {ios} like Mac OS X) AppleWebKit/{webkit} (KHTML, like Gecko) Version/{safari} Mobile/15E148 Safari/{webkit}'
                template = template.replace('{ios}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{webkit}', str(self.browser_version.webkit))
                template = template.replace('{safari}', self.browser_version.format(partitions=2))
                return template
            if self.browser == 'firefox':
                template = 'Mozilla/5.0 (iPhone; CPU iPhone OS {ios} like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) FxiOS/{firefox} Mobile/15E148 Safari/605.1.15'
                template = template.replace('{ios}', self.platform_version.format(partitions=3, separator='_', trim_zero=True))
                template = template.replace('{firefox}', self.browser_version.format(partitions=2))
                return template

        raise exceptions.CannotGenerateError(self)



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator


class TestDevice(unittest.TestCase):
    def test_device(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device=('desktop'))
            self.assertTrue(ua.device == 'desktop')

    def test_device_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device=('desktop', 'mobile'))
            self.assertTrue(ua.device == 'desktop' or ua.device == 'mobile')

    def test_device_3(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='desktop')
            self.assertTrue(ua.device == 'desktop')

    def test_device_4(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='mobile')
            self.assertTrue(ua.device == 'mobile')

    def test_device_5(self):
        for i in range(0, 100):
            ua = ua_generator.generate(device='desktop')
            self.assertFalse(ua.device == 'mobile')

    def test_device_6(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('ios'))
            self.assertTrue(ua.device == 'mobile')

    def test_device_7(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform=('windows', 'macos'))
            self.assertTrue(ua.device == 'desktop')


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..version import Version, VersionRange
from ...options import Options

# https://developer.apple.com/documentation/ios-ipados-release-notes
# https://developer.apple.com/news/releases/
# https://support.apple.com/en-us/HT201222
versions: List[Version] = [
    Version(major=14, minor=0, build=(0, 1)),
    Version(major=14, minor=1, build=0),
    Version(major=14, minor=2, build=(0, 1)),
    Version(major=14, minor=3, build=0),
    Version(major=14, minor=4, build=(0, 2)),
    Version(major=14, minor=5, build=(0, 1)),
    Version(major=14, minor=6, build=0),
    Version(major=14, minor=7, build=(0, 1)),
    Version(major=15, minor=0, build=(0, 2)),
    Version(major=15, minor=1, build=(0, 1)),
    Version(major=15, minor=2, build=(0, 1)),
    Version(major=15, minor=3, build=(0, 1)),
    Version(major=15, minor=4, build=(0, 1)),
    Version(major=15, minor=5, build=0),
    Version(major=15, minor=6, build=(0, 1)),
    Version(major=16, minor=4, build=(0, 1)),
    Version(major=16, minor=5, build=(0, 2)),
    Version(major=16, minor=6, build=(0, 1)),
    Version(major=17, minor=0, build=(0, 3)),
    Version(major=17, minor=1, build=(0, 2)),
    Version(major=17, minor=2, build=(0, 1)),
    Version(major=17, minor=3, build=(0, 1)),
    Version(major=17, minor=4, build=(0, 1)),
    Version(major=17, minor=5, build=(0, 1)),
    Version(major=17, minor=6, build=(0, 1)),
    Version(major=18, minor=0, build=(0, 1)),
    Version(major=18, minor=1, build=(0, 1)),
    Version(major=18, minor=2, build=(0, 1)),
    Version(major=18, minor=3, build=(0, 1)),
]


def get_version(options: Options) -> Version:
    if options.version_ranges is not None and 'ios' in options.version_ranges:
        if type(options.version_ranges['ios']) == VersionRange:
            filtered = options.version_ranges['ios'].filter(versions)
            if type(filtered) == list and len(filtered) > 0:
                return random.choice(filtered)

    weights = None
    if options.weighted_versions:
        weights = [1.0] * len(versions)
        weights[-1] = 10.0
        weights[-2] = 9.0
        weights[-3] = 8.0
        weights[-4] = 7.0

    choice: List[Version] = random.choices(versions, weights=weights, k=1)
    return choice[0]
